#!/usr/bin/python
#  -*-python-*-

# configScreen
# Author: Nathan Wilson / Greg Underwood / Scott Chapin

# configScreen allows users to change their X-windows screen configuration.
# In particular it rewrites files in /etc/X11.  For each file matching
# /etc/X11/*/*.h, the file /etc/X11/* is written where the * matches the
# second *.  The *.h files were created to work with configScreen.  Each of
# the *.h files describe all the desired forms of the corresponding file
# using C-preprocessor directives.
#
# The *.h files also describe the XConfigScreen interface in a set of C++
# style comments at the end of each file.  The top second of each file
# lists the interface state that was used to generate the given file.
# See the /etc/X11/X0screens.h for an example and more details on the format
# and how to extend the file to allow for more tweaks to the X0screens file.

from __future__ import nested_scopes
import sys, os, re, string
from Tkinter import *

# getTargetFiles - returns a list of [source,dest] pairs of files.
# source is of the form <dir>/<subdir>/<filename>.h and the dest
# is of the form <dir>/<filename>
def getTargetFiles():
	result = []
	if os.getuid() == 0:
		lines = os.popen('ls /etc/X11/*/*.h', 'r').readlines()
	else:
		lines = os.popen('ls ./*/*.h', 'r').readlines()

	# Breakup the line into the base directory, the subdirectory (which
	# is ignored) and the filename
	pattPrefix = re.compile('^(.*/)*[^/]+/([^/]+)\.h$')
	for line in lines:
		match = pattPrefix.match(line)
		if match != None:
			pair = [line[:-1], match.group(1) + match.group(2)]
			result.append(pair)
		else:
			sys.stderr.write("Unable to find prefix for " + line)
			return None
	return result


# readOptions - reads the current settings stored in the given file
def readOptions(filename):
	pattOption = re.compile('^#+_([A-Z_]+=\d+)')
	oldPattOption = re.compile('^#([A-Z_]+=\d+)')
	result = []
	if os.access(filename, os.R_OK):
		for line in open(filename, 'r').readlines():
			match = pattOption.match(line)
			if match != None:
				result.append(match.group(1))
			else:
				match = oldPattOption.match(line)
				if match != None:
					result.append(match.group(1))
	return result

def printOptions():
	filepairs = getTargetFiles()
	if filepairs:
		for filepair in filepairs:
			options = readOptions(filepair[1])
			if options:
				print "Options for " + filepair[1]
				for opt in options:
					print opt
			else:
				print filepair[1] + " has never been configured with configScreen"

def constructPrefix(cmd_prefix, filename):
	success = 1
	pattName = re.compile('^([A-Z_]+)=')
	hits = {}
	for v in sys.argv[1:]:
		match = pattName.match(v)
		if match != None:
			hits[match.group(1)] = v
			cmd_prefix = cmd_prefix + " -D" + v
		else:
			sys.stderr.write("Unable to parse arg: " + v + "\n")
			success = 0
	defaults = readOptions(filename)
	for d in defaults:
		match = pattName.match(d)
		if match != None:
			if not hits.has_key(match.group(1)):
				cmd_prefix = cmd_prefix + " -D" + d
		else:
			sys.stderr.write("Unable to parse default: " + d + "\n")
			success = 0
	cmd_prefix = cmd_prefix + " "
	if not success:
		cmd_prefix = ""
	return cmd_prefix

# writeFile - creates the new <file> by running */<file>.h
# through the C-preprocessor with the macros currently set in the
# various Categorys.  Tries to be a bit cautious by outputing to
# a tmp file, checking the status and then only overwriting the
# actual file if the command succeeded
def writeFile():
	success = 0
	target_filepairs = getTargetFiles()
	if target_filepairs:
		for filepair in target_filepairs:
			success = runCommand(filepair)
			if not success:
				return 0
	return success

def runCommand(filepair):
	success = 1
	hFile = filepair[0]
	filename = filepair[1]

	cmd_prefix = constructPrefix("gcc -E -P", filename)
	
	tmpFile = filename + ".tmp"
	command = cmd_prefix + hFile + " | grep -v '^ *$' > " + tmpFile
	if 0 != os.system(command):
		sys.stderr.write("Unable to create new " + filename)
		success = 0
	elif 0 != os.system("chmod 644 " + tmpFile):
		sys.stderr.write("Unable to make " + tmpFile +
				 " world readable")
		success = 0
	elif 0 != os.system("mv " + tmpFile + " " + filename):
		sys.stderr.write("Unable to move " + tmpFile + " to " +
				 filename)
		success = 0
	return success

# validMachine - Makes sure we're running on a machine of the expected
# type.
def validMachine():
	lines = os.popen('uname', 'r').readlines()
	return lines[0][:5] == 'Linux'

# Check that there are no args.  If there are then a help message is
# printed.
do_help = 1
if len(sys.argv) > 1:
	do_help = sys.argv[1] == '-h' or sys.argv[1] == "-help"
if do_help:
	print "configScreen version 1.45"
	print "Created by Nathan Wilson - maintained by Scott Chapin"
	print "Copyright 2001-2004 Dreamworks SKG, LLP. All Rights Reserved."
	print "Usage: configScreen <monitor-options>"
	print
	printOptions()
else:
	if validMachine():
		if writeFile():
			if 0 != os.system("killall -s 9 X Xorg"):
				sys.stderr.write("Unable to restart X.\n" +
					"Please hit Ctrl-Alt-Backspace")
			else:
				sys.exit(0)
	sys.exit(1)
